<?php

namespace SGalinski\SgJobs\Routing\Aspect;

/**
 *
 * Copyright notice
 *
 * (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 * All rights reserved
 *
 * This script is part of the TYPO3 project. The TYPO3 project is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 *
 * This script is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This copyright notice MUST APPEAR in all copies of the script!
 */

use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Routing\Aspect\PersistedMappableAspectInterface;
use TYPO3\CMS\Core\Routing\Aspect\StaticMappableAspectInterface;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class JobAspect
 *
 * @deprecated @TODO Drop it in one of the next breaking versions. It should not be used anymore. Look at the README. We switched already to the path_segment field.
 *
 * @package SGalinski\SgJobs\Routing\Aspect
 * @author Kevin Ditscheid <kevin.ditscheid@sgalinski.de>
 */
class JobTitleMapper implements PersistedMappableAspectInterface, StaticMappableAspectInterface {

	/**
	 * Generate the job title from the job id given to this function as the $value parameter
	 *
	 * @param string $value
	 * @return string|null
	 * @throws \Exception
	 */
	public function generate(string $value): ?string {
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)
			->getQueryBuilderForTable('tx_sgjobs_domain_model_job');
		$job = $queryBuilder->select('job.uid', 'job.title')
			->from('tx_sgjobs_domain_model_job', 'job')
			->where(
				$queryBuilder->expr()->eq('job.uid', $queryBuilder->createNamedParameter($value, \PDO::PARAM_INT))
			)
			->setMaxResults(1)
			->execute()->fetch();

		return \urlencode(
			\preg_replace('/[^a-zA-Z0-9_]/', '', \str_replace([' ', '-'], '_', \strtolower($job['title'])))
			. '-' . $job['uid']
		);
	}

	/**
	 * Resolve a job title given by the $value parameter to a job id. It is slicing the string into the various parts
	 * via regular expression and searches the database for a record, that matches the parts.
	 *
	 * @param string $value
	 * @return string|null
	 */
	public function resolve(string $value): ?string {
		if (!\preg_match('/^(?P<title>.+)-(?P<uid>.+)/', $value, $matches)) {
			return NULL;
		}

		$jobUid = (int) $matches['uid'];
		return ($jobUid > 0 ? $jobUid : NULL);
	}
}
